/*
 * Copyright (c) LINAGORA
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 * 1) Redistributions of source code must retain the above 
 * copyright notice, this list of conditions and the 
 * following  disclaimer.
 * 2)  Redistributions in binary form must reproduce the 
 * above copyright notice, this list of conditions and 
 * the following disclaimer in the documentation and/or 
 * other materials provided with the distribution.
 * 3) Neither the name of "Rafael Steil" nor 
 * the names of its contributors may be used to endorse 
 * or promote products derived from this software without 
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT 
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 * 
 * Created on 27.03.2007
 */
package net.jforum.dao.informix;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.jforum.JForumExecutionContext;
import net.jforum.dao.generic.GenericTopicDAO;
import net.jforum.entities.Topic;
import net.jforum.entities.User;
import net.jforum.exceptions.DatabaseException;
import net.jforum.repository.ForumRepository;
import net.jforum.util.DbUtils;
import net.jforum.util.preferences.ConfigKeys;
import net.jforum.util.preferences.SystemGlobals;

/**
 * @author Jonathan Clarke <jclarke@linagora.com>
 */
public class InformixTopicDAO extends GenericTopicDAO
{
	/**
	 * @see net.jforum.dao.TopicDAO#selectAllByForumByLimit(int, int, int)
	 */
	public List selectAllByForumByLimit(int forumId, int startFrom, int count)
	{
		String sql = SystemGlobals.getSql("TopicModel.selectAllByForumByLimit");
		sql = sql.replaceAll("#N#", String.valueOf(startFrom + count));

		PreparedStatement p = null;
		ResultSet rs = null;
		try {
			p = JForumExecutionContext.getConnection().prepareStatement(sql);
			p.setInt(1, forumId);

			rs = p.executeQuery();
			
			// jump over first unwanted records
			for (int i=1; i < startFrom; i++) rs.next();
			
			List list = this.fillTopicsData(rs);
			p = null;
			return list;
		}
		catch (SQLException e) {
			throw new DatabaseException(e);
		}
		finally {
			DbUtils.close(rs, p);
		}
	}

	/**
	 * @see net.jforum.dao.TopicDAO#selectByUserByLimit(int, int, int)
	 */
	public List selectByUserByLimit(int userId, int startFrom, int count)
	{
		PreparedStatement p = null;
		ResultSet rs = null;
		try {
			String sql = SystemGlobals.getSql("TopicModel.selectByUserByLimit");
			sql = sql.replaceAll(":fids:", ForumRepository.getListAllowedForums());
			sql = sql.replaceAll("#N#", String.valueOf(startFrom + count));
			p = JForumExecutionContext.getConnection().prepareStatement(sql);

			p.setInt(1, userId);

			rs = p.executeQuery();
			
			// jump over first unwanted records
			for (int i=1; i < startFrom; i++) rs.next();
			
			List list = this.fillTopicsData(rs);
			p = null;
			return list;
		}
		catch (SQLException e) {
			throw new DatabaseException(e);
		}
		finally {
			DbUtils.close(rs, p);
		}
	}

	/**
	 * Fills all topic data. The method will try to get all fields from the topics table, as well
	 * information about the user who made the first and the last post in the topic. <br>
	 * <b>The method <i>will</i> close the <i>PreparedStatement</i></b>
	 * 
	 * @param p the PreparedStatement to execute
	 * @return A list with all topics found
	 * @throws SQLException
	 */
	public List fillTopicsData(ResultSet rs)
	{
		List l = new ArrayList();
		PreparedStatement p = null;
		
		try {
			SimpleDateFormat df = new SimpleDateFormat(SystemGlobals.getValue(ConfigKeys.DATE_TIME_FORMAT));

			StringBuffer sbFirst = new StringBuffer(128);
			StringBuffer sbLast = new StringBuffer(128);

			while (rs.next()) {
				Topic t = this.getBaseTopicData(rs);

				// Posted by
				User u = new User();
				u.setId(rs.getInt("user_id"));
				t.setPostedBy(u);

				// Last post by
				u = new User();
				u.setId(rs.getInt("last_user_id"));
				t.setLastPostBy(u);

				t.setHasAttach(rs.getInt("attach") > 0);
				t.setFirstPostTime(df.format(rs.getTimestamp("topic_time")));
				t.setLastPostTime(df.format(rs.getTimestamp("post_time")));
				t.setLastPostDate(new Date(rs.getTimestamp("post_time").getTime()));

				l.add(t);

				sbFirst.append(rs.getInt("user_id")).append(',');
				sbLast.append(rs.getInt("last_user_id")).append(',');
			}

			rs.close();
			rs = null;
			
			// Users
			if (sbFirst.length() > 0) {
				sbLast.delete(sbLast.length() - 1, sbLast.length());

				String sql = SystemGlobals.getSql("TopicModel.getUserInformation");
				sql = sql.replaceAll("#ID#", sbFirst.toString() + sbLast.toString());

				Map users = new HashMap();

				p = JForumExecutionContext.getConnection().prepareStatement(sql);
				rs = p.executeQuery();

				while (rs.next()) {
					users.put(new Integer(rs.getInt("user_id")), rs.getString("username"));
				}

				rs.close();
				rs = null;
				p.close();
				p = null;

				for (Iterator iter = l.iterator(); iter.hasNext();) {
					Topic t = (Topic) iter.next();
					t.getPostedBy().setUsername((String) users.get(new Integer(t.getPostedBy().getId())));
					t.getLastPostBy().setUsername((String) users.get(new Integer(t.getLastPostBy().getId())));
				}
			}

			return l;
		}
		catch (SQLException e) {
			throw new DatabaseException(e);
		}
		finally {
			DbUtils.close(rs, p);
		}
	}
	
	/**
	 * @see net.jforum.dao.TopicDAO#selectRecentTopics(int)
	 */
	public List selectRecentTopics(int limit)
	{
		PreparedStatement p = null;
		try {
			String sql = SystemGlobals.getSql("TopicModel.selectRecentTopicsByLimit");
			sql = sql.replaceAll("#N#", String.valueOf(limit));
			p = JForumExecutionContext.getConnection().prepareStatement(sql);

			List list = this.fillTopicsData(p);
			p = null;
			return list;
		}
		catch (SQLException e) {
			throw new DatabaseException(e);
		}
		finally {
			DbUtils.close(p);
		}
	}
}